using namespace arma;

class Kalman {
private:
  mat A, H, Q, R, xest, pest;
  double dt;

public:
  // constructor, sets up data structures
  Kalman() : dt(1.0) {
    A.eye(6,6);
    A(0,2) = A(1,3) = A(2,4) = A(3,5) = dt;
    H.zeros(2,6);
    H(0,0) = H(1,1) = 1.0;
    Q.eye(6,6);
    R = 1000 * eye(2,2);
    xest.zeros(6,1);
    pest.zeros(6,6);
  }

  // sole member function: estimate model
  mat estimate(const mat & Z) {
    unsigned int n = Z.n_rows, k = Z.n_cols;
    mat Y = zeros(n, k);
    
    for (unsigned int i = 0; i<n; i++) {
      colvec z = Z.row(i).t();
      
      // predicted state and covriance
      mat xprd = A * xest;
      mat pprd = A * pest * A.t() + Q;
      
      // estimation
      mat S = H * pprd.t() * H.t() + R;
      mat B = H * pprd.t();

      mat kg = trans(solve(S, B));

      // estimated state and covariance
      xest = xprd + kg * (z - H * xprd);
      pest = pprd - kg * H * pprd;
      
      // compute the estimated measurements
      colvec y = H * xest;
      
      Y.row(i) = y.t();
    }
    return Y;
  }
};
